// MemBrain C Code export library V06.00.00.00

/*
--- License information and liability disclaimer ---
------------------
The source code exported by MemBrain including all supplementary source code
provided as part of the MemBrain installation is subject to licensing according
to the terms indicated in the license agreement that is displayed when
installing the MemBrain software.

You can also download and view this license agreement on the MemBrain
homepage (http://www.membrain-nn.de).

The terms of the license agreement do especially prohibit the commercial use of
any of the MemBrain source code items for a commercial purpose except the
user or company that intends to use the code in a commercial way is a
commercially registered MemBrain user and the owner of a corresponding commercial
MemBrain license key.

The MemBrain program and all associated supplementary software including but not
limited to the MemBrain DLL and all MemBrain generated source code are distributed
"as is". No warranty of any kind is expressed or implied. You use it at your own risk.
Thomas Jetter will not be liable for data loss, damages and loss of profits or
properties or any other kind of damages or loss caused by use or misuse of this
software.

Copyright 2003 - 2019 Thomas Jetter
*/

#include "NeuralNetDef.h"

/// Get the current output signal of the given link
MB_FLOAT NeuralLinkGetOutput(T_LINK_IDX index)
{
    MB_FLOAT out;
    const SNeuralLinkParms* pLink = &NEURAL_LINK_PARMS[index];

#ifdef LINK_LENGTHS
    if (pLink->len > 1)
    {
        // get delayed signal fom neuron if link has length > 1
        out = NeuronGetPastOutput(pLink->sourceIdx, (T_LINK_LEN)(pLink->len - 1));
    }
    else
    {
        out = NeuronGetLatestOutput(pLink->sourceIdx);
    }
#else
    out = NeuronGetLatestOutput(pLink->sourceIdx);
#endif

    // Return neuron output multiplied by link weight
    return out * pLink->weight;
}


#ifdef ACT_FUNC_MIN_EUCLID_DIST
/// Get the neuron output signal that lead to the current output signal of the link
MB_FLOAT NeuralLinkGetInNeuronOutput(T_LINK_IDX index)
{
    MB_FLOAT out;
    const SNeuralLinkParms* pLink = &NEURAL_LINK_PARMS[index];

#ifdef LINK_LENGTHS
    if (pLink->len > 1)
    {
        // get delayed signal fom neuron if link has length > 1
        out = NeuronGetPastOutput(pLink->sourceIdx, (T_LINK_LEN)(pLink->len - 1));
    }
    else
    {
        out = NeuronGetLatestOutput(pLink->sourceIdx);
    }
#else
    out = out = NeuronGetLatestOutput(pLink->sourceIdx);
#endif

    return out;
}
#endif


