// MemBrain C Code export library V06.00.00.00

/*
--- License information and liability disclaimer ---
------------------
The source code exported by MemBrain including all supplementary source code
provided as part of the MemBrain installation is subject to licensing according
to the terms indicated in the license agreement that is displayed when
installing the MemBrain software.

You can also download and view this license agreement on the MemBrain
homepage (http://www.membrain-nn.de).

The terms of the license agreement do especially prohibit the commercial use of
any of the MemBrain source code items for a commercial purpose except the
user or company that intends to use the code in a commercial way is a
commercially registered MemBrain user and the owner of a corresponding commercial
MemBrain license key.

The MemBrain program and all associated supplementary software including but not
limited to the MemBrain DLL and all MemBrain generated source code are distributed
"as is". No warranty of any kind is expressed or implied. You use it at your own risk.
Thomas Jetter will not be liable for data loss, damages and loss of profits or
properties or any other kind of damages or loss caused by use or misuse of this
software.

Copyright 2003 - 2019 Thomas Jetter
*/

#include "NeuralNetDef.h"
#ifdef NEED_FLOAT_RAND
#include "Random.h"
#endif

/// Initialize and reset the neural net. Call this function at least once in first place!!
void NeuralNetInitAndReset(void)
{
    T_NEURON_COUNT i;

    // reset every neuron
    for (i = 0; i < MB_NEURON_COUNT; i++)
    {
        NeuronReset(i);
    }

#ifdef FIRE_THRESHOLDS
    // initialize random unit
    RandomInit(0);
#endif
}


/// Let the net perform one think step
void NeuralNetThinkStep(void)
{
#ifdef MB_ACT_SUM_POST_PROC_LAYER_COUNT
    T_LAYER_COUNT actSumLayerIdx = 0;
    const SLayerDef* pLayerDef = &ACT_SUM_POST_PROC_LAYERS[actSumLayerIdx];
    T_NEURON_COUNT actSumLayerStartIdx = pLayerDef->startIdx;
    T_NEURON_COUNT actSumLayerEndIdx = pLayerDef->endIdx;
    T_ACT_FLOAT layerSum;
    BOOL inActSumLayer = FALSE;
    BOOL postProcess = FALSE;
    T_NEURON_COUNT j;
#endif
    T_NEURON_COUNT i;

    // cause every neuron to propagate
    for (i = 0; i < MB_NEURON_COUNT; i++)
    {
        NeuronPropagate(i);
#ifdef MB_ACT_SUM_POST_PROC_LAYER_COUNT
        if (inActSumLayer == FALSE)
        { // Not yet entered post processing layer
            if (i == actSumLayerStartIdx)
            { // Entering post processing layer
                layerSum = NeuronGetActivation(i);
                inActSumLayer = TRUE;
            }
        }
        else
        { // Already within a post processing layer
            layerSum += NeuronGetActivation(i);
        }
        // Finished post porcessing layer (layer contains only one neuron)?
        postProcess = (i == actSumLayerEndIdx) ? TRUE : FALSE;

        if (postProcess)
        {
            for (j = actSumLayerStartIdx; j <= actSumLayerEndIdx; j++)
            {
                NeuronPostProcess(j, layerSum);
                NeuronFire(j);
            }
            if (actSumLayerIdx < MB_ACT_SUM_POST_PROC_LAYER_COUNT - 1)
            {
                actSumLayerIdx++;
                pLayerDef = &ACT_SUM_POST_PROC_LAYERS[actSumLayerIdx];
                actSumLayerStartIdx = pLayerDef->startIdx;
                actSumLayerEndIdx = pLayerDef->endIdx;
            }
        }
        if (inActSumLayer == FALSE)
        { // Outside of post processing layer we can fire (calculate neuron output signal) immediately.
          // Else this is part of the post layer processing above.
#endif
            NeuronFire(i);
#ifdef MB_ACT_SUM_POST_PROC_LAYER_COUNT
        }
        if (postProcess == TRUE)
        { // Finished a post processing layer. Leaving this layer now with the next loop.
            inActSumLayer = FALSE;
        }
#endif
    }
}

#if MB_IN_NEURON_COUNT != 0
/// Apply an activation value to an input neuron. <idx> can range from 0 to MB_IN_NEURON_COUNT - 1
BOOL NeuralNetApplyInputAct(T_NEURON_COUNT inputIdx, T_ACT_FLOAT act)
{
    BOOL success;

    if (inputIdx < MB_IN_NEURON_COUNT)
    {
        NeuronSetActivation(inputIdx, act);
        success = TRUE;
    }
    else
    {
        success = FALSE;
    }

    return success;
}
#endif

/// Get the activation value of an output neuron. <idx> can range from 0 to MB_OUT_NEURON_COUNT - 1
BOOL NeuralNetGetOutputAct(T_NEURON_COUNT outputIdx, T_ACT_FLOAT* pAct)
{
    BOOL success;

    if (outputIdx < MB_OUT_NEURON_COUNT)
    {
        *pAct = NeuronGetActivation((T_NEURON_COUNT)(outputIdx + MB_IN_NEURON_COUNT + MB_HID_NEURON_COUNT));
        success = TRUE;
    }
    else
    {
        success = FALSE;
    }

    return success;
}

/// Get the output value of an output neuron. <idx> can range from 0 to MB_OUT_NEURON_COUNT - 1
BOOL NeuralNetGetOutputOut(T_NEURON_COUNT outputIdx, MB_FLOAT* pOut)
{
    BOOL success;

    if (outputIdx < MB_OUT_NEURON_COUNT)
    {
        *pOut = NeuronGetLatestOutput((T_NEURON_COUNT)(outputIdx + MB_IN_NEURON_COUNT + MB_HID_NEURON_COUNT));
        success = TRUE;
    }
    else
    {
        success = FALSE;
    }

    return success;
}


